# from net import * 
from neuron import h
# h.nrn_load_dll("x86_64/.libs/libnrnmech.so")
import numpy as np
h.load_file('stdrun.hoc')

# Declarations
nclist = h.List()
rdm = [h.Random() for ii in xrange(2)]
veclist = [h.Vector() for ii in xrange(10)]
scr = [h.Vector() for ii in xrange(2)]
animv = [h.Vector() for ii in xrange(2)]
tvec, ind, drv = h.Vector(), h.Vector(), h.Vector() # time, index, drawing vectors
sl=h.SectionList() # empty list

# Cell template
class Cell:
  def __init__ (self, _id, _x, _y, _z):
      self.pp = h.IntervalFire()
      self.pp.tau = 10
      self.pp.invl = 20
      self.id, self.x, self.y, self.z = _id, _x, _y, _z
  def is_art ():
      return 1
  def connect2target (targ,nc):
      nc=h.NetCon(pp, targ)
      return nc
  def M ():
      return pp.M()

# Network specification
# createnet()
def createnet ():
  global cells
  cells = [Cell(i,i,0,0) for i in xrange(ncell)]
  wire()

# wire():: full non-self connectivity
### artificial cell templates have obj.pp
### params: ncell
### creates nclist: list of NetCons
def wire ():
  nclist.remove_all()
  for pre in cells: 
    for post in cells: 
      if pre!=post: nclist.append(h.NetCon(pre.pp,post.pp))

# ranwire()
def ranwire ():
  nclist.remove_all()
  rdm[0].discunif(0,ncell-1)
  for i in xrange(ncell):
    proj=rdm[0].repick()
    if proj<ncell/2:
      beg=proj
      end=proj+int(ncell/2.0)-1 
    else:
      beg=proj-int(ncell/2.0)+1
      end=proj
    for j in xrange(beg,end+1):
      if i!=j:
        netcon = h.NetCon(cells[i].pp,cells[j].pp)
        nclist.append(netcon)

# parameters
h.tstop = 500
ncell = 10
ta=10
w=-0.01
delay=4
low=10
high=11

# routines: weight(),delay(),tau(),interval()
def weight (w=-0.3): 
  if not isinstance(w,float):
    vv=h.Vector()
    for nc in nclist: vv.append(nc.weight[0])
    print vv.min(),vv.max(),vv.mean(),vv.stdev()
  else:
    for n in nclist: n.weight[0] = w

# weight2(WT,EXCLUDE_VEC) :: set weight to WT 
# unless in EXCLUDE_VEC then set wt. to 0
def weight2 (w,ecv):
  for i,nc in enumerate(nclist):
    if ecv.contains(i): 
      nc.weight[0]=0
    else:               
      nc.weight[0]=w

def setdelay (delay):
  for n in nclist: n.delay = delay

def settau (tau): 
  for c in cells: c.pp.tau = tau

# interval(low,high) randomly sets cells to have periods between low and high
def interval (low,high):
  rdm[0].uniform(low,high)
  veclist[0].resize(ncell)
  veclist[0].setrand(rdm[0])
  for x,ce in zip(veclist[0],cells):
    ce.pp.invl = x

# setparams() sets weight, delay, tau and intervals
def setparams (w=-0.2,delay=2,ta=10,low=10,high=12):
  weight(w)
  setdelay(delay)
  settau(ta)  
  interval(low, high)

# Run code
# savspks() -- save to a vector
def savspks ():
  for v in animv: v.resize(0)
  for c in cells:
    if h.cvode.netconlist(c, '', '').count()==0:
      tmpobj=h.NetCon(c.pp, None)
    else:
      tmpobj=cvode.netconlist(c, '', '').object(0)
    tmpobj.record(tvec,ind)
    animv[0].append(objnum(tmpobj))
    animv[1].append(objnum(tmpobj.pre)) # in this case are all in a row anyway

# showspks() -- show spikes on graph g
def markspks ():
  g.erase()
  ind.mark(g,tvec,"O",4,2,1)
  g.flush()
  g.exec_menu("View = plot")

def showspks ():
  # ind.mark(g,tvec,"O",8,2,1)
  for x in tvec:
    g.beginline(3,1)
    g.line(x,ind.min())
    g.line(x,ind.max()) 
  g.flush()
  g.exec_menu("View = plot")

# syncer() :: returns sync measure 0 to <1
def syncer (vec=tvec,width=1):
  '''measures how well spikes "fill up" the time
  assumes spike times in tvec, tstop
  param: width
  syncer doesn't take account of prob of overlaps 
  due to too many spikes stuffed into too little time'''
  t0=-1; cnt=0
  for tt in vec:
    if tt>=t0+width: t0=tt; cnt+=1
  return 1.0-cnt/(h.tstop/width)

# autorun() changes weights
def autorun ():
  savevec() # clear the storage vectors
  for w in [-0.5,-0.3,-0.1,-0.01,-0.001]:
    weight(w)
    run()
    savevec([ind,tvec])
    veclist[1].append(w)
    s = syncer()
    veclist[2].append(s)
    print w,s
  g.erase()
  veclist[2].plot(g,veclist[1])
  veclist[2].mark(g,veclist[1],"O",8,2,1)
  g.exec_menu("View = plot")

# autorun1() changes connections density
def autorun1 ():
  savevec()
  maxx, pij_inc = -0.1, 0.1
  S=ncell*(ncell-1)
  for i in [1,2,3]: veclist[i].resize(0)
  for ii in xrange(10):
    C = (1-ii*pij_inc) #  percent convergence
    w=maxx/C  #  scale weight up as convergence goes down
    setparams()
    weight2(w,veclist[3])
    run()
    savevec([ind,tvec])
    print S-veclist[3].size(),syncer()
    veclist[1].append((S-veclist[3].size())/S)
    veclist[2].append(syncer())
    rdmunq(veclist[3],0.1*S,S) # increase those set to 0
  g.erase()
  veclist[2].plot(g,veclist[1])
  veclist[2].mark(g,veclist[1],"O",8,2,1)
  g.exec_menu("View = plot")

# Utility functions
def objnum (ob):
  "Return index of internal represetation of a NEURON object"
  hn = ob.hname()
  return int(hn[hn.find('[')+1:hn.find(']')])

## savevec
def savevec (vecs=None):
  '''savevec(list of vectors) add vectors onto datavecs list'''
  global datavecs
  try: datavecs
  except: datavecs = [] # initialize if doesn't exist
  if vecs==None: 
    datavecs = [] # clear
  else: 
    datavecs.extend([v.c() for v in vecs])

## setdensity(pij) sets connection density to 0<pij<1
def setdensity (pij):
  maxx=-0.1
  S=nclist.count()
  veclist[3].resize(0)
  rdmunq(veclist[3],(1-pij)*S,S) #  number to set to 0
  C = (1-pij) #  percent convergence
  w=maxx/C  #  scale weight up as convergence goes down
  setparams()
  weight2(w,veclist[3])

## rdmunq(vec,n,max)
def rdmunq (vec,n,max):
  '''augment vec by n unique vals from a rdm providing discrete uniform integers from 0 to max-1'''
  import random
  sz = int(vec.size())
  if sz+n > max: raise Exception('rdmunq ERROR: requesting %d uniques values but only %d cells\n'%(sz+n, max))
  if (sz+n==max):  # all values
    vec.resize(max)
    vec.indgen()
  else:
    # make set the ones that are not already in vec
    unused = set(range(max)) - set(vec)
    nun = random.sample(unused,int(n))
    for x in nun: vec.append(x)    

## rdmord (vec,n) randomly ordered numbers 0->n-1 in vec
def rdmord (vec,n):
  rdm[0].uniform(0,100)
  scr[0].resize(n)
  scr[0].setrand(rdm[0])
  scr[0].sortindex(vec)

## vcount (num,vec)
def vcount (num,vec):
  scr[0].where(vec,"==",num)
  return scr[0].size()

## Mapping functions
##  getcnum(CELL_OBJ) return index given cell object
def getcnum (cell):  return cell.id

## fconn(PREVEC,POSTVEC) places values of 
def fconn (prev,post):
  '''pre- and post-syn cells in parallel vectors
  only lists pairs with non-zero connections; getcnum() returns index of cell obj'''
  prev.resize(0)
  postv.resize(0)
  for ii in xrange(0,nclist.count()):
    XO=nclist.o(ii)
    if XO.weight[0]!=0:
      prev.append(getcnum(XO.pre))
      postv.append(getcnum(XO.syn))

## showconns() -- show all the connections as line segments
def showconns ():
  g.erase()
  fconn(scr[0],scr[1])
  for ii in xrange(0,scr[0].size()):
    pr=scr[0].x[ii]
    po=scr[1].x[ii]
    drawline(pr,po,10)
  g.flush()

## showconv1(ID,color) -- show convergence to one cell as line seg
def showconv1 (ID,colr=2):
  fconn(scr[0],scr[1])
  for ii in xrange(0,scr[0].size()):
    pr=scr[0].x[ii]
    po=scr[1].x[ii]
    if po==ID:
      drawline(pr,po,10,colr,3)
      print pr
  print ''
  g.flush()

## showdiv1() -- show divergence from one cell as line seg
def showdiv1 (id,colr=2):
  fconn(scr[0],scr[1])
  for ii in xrange(0,scr[0].size()):
    pr=scr[0].x[ii]
    po=scr[1].x[ii]
    if pr==id:
      drawline(pr,po,10,colr,8)
      print po
  print ''
  g.flush()

## xpos, ypos
def xpos (x,cols): return x%cols
def ypos (co1s,y): return int(co1s/y)

## func distn () calc distance
def distn (c1,c2,cols):
  from math import sqrt
  xd=xpos(c1,cols)-xpos(c2,cols)
  yd=ypos(c1,cols)-ypos(c2,cols)
  return sqrt(xd*xd+yd*yd)

## distwire(pij)
def distwire (pij):
  allsyns = nclist.count()
  total=pij*allsyns # how many syns to set
  rdm[1].uniform(0,1) # for flipping coin
  #  maxdist==12.728 for 10x10; mindist=1 (neighbors)
  maxdist=0.33*distn(0,ncell-1,sqrt(ncell)) # the full dist from lower left to upper right
  maxwt=-0.9/pij  # norm wt by convergence
  loop=cnt=0 # counters
  for i in xrange(0,allsyns): nclist.o(i).weight[0] = 0 # # clear weights
  while cnt<total and loop<4:
    rdmord(veclist[3],allsyns)  # # test each synapse in random order
    for ii in xrange(0,veclist[3].size()):
      XO=nclist.object(veclist[3].x[ii]) # # pick a synapse
      # max prob of connection is 0.8*(1-mindist/maxdist)~74% for 10x10
      # zero prob of diag connection from corner to corner
      prob = 1.0*(1-(distn(getcnum(XO.pre),getcnum(XO.syn),sqrt(ncell))/maxdist))
      if rdm[1].repick<prob:
        XO.weight=maxwt
        cnt+=1
      if cnt>=total:
        break # finished
  print cnt,total
  if cnt<total:
    print "distwire ERR: target ", total, "set ", cnt
    
# drawline(beg,end,columns[,color,line_width]) 
def drawline (beg,end,cols,clr=4,lwid=1): 
  # local beg,end,cols,clr,lwid
  g.beginline(clr,lwid)
  g.line(xpos(beg,cols),ypos(beg,cols))
  g.line(xpos(end,cols),ypos(end,cols))

# Animation
## animplot() put up the shape plot for hinton diagram
def animplot ():
  global gx
  gx = h.PlotShape(sl,0)
  h.flush_list.append(gx)
  ctern(gx,0,2)
  gx.view(1,0,1.1,1.1,500,200,100,100)
  drawcells()

## Ternary color map
def ctern (cm,minc,maxc):
  cm.colormap(3)
  cm.colormap(0, 255, 0, 0)
  cm.colormap(1, 255, 255, 0)
  cm.colormap(2, 0, 0, 255)
  cm.scale(minc, maxc)

## drawcells() draw squares of hinton diagram
def drawcells ():
  if ncell!=100: raise Exception('ERROR: drawcells() currently written for ncell=100')
  gx.erase_all()
  drv.resize(ncell)
  xoff=1.1
  yoff=0.1
  wdt=0.1
  nx=ny=10
  for i in xrange(0,nx):
    for j in xrange(0,ny):
      gx.hinton(drv._ref_x[j*nx+i],(i+.5)*wdt+xoff,(j+.5)*wdt+yoff,wdt)
  gx.size(1, 2.2, 0, 1.2)
  gx.exec_menu("Shape Plot")

## chkhint(cell#) light up a location for a single cell
def chkhint (cellid):
  drv.fill(0)
  drv.x[cellid]=2
  gx.flush()

## anim() animates sim stored in tvec,ind
def anim ():
  tstep=0.1
  sz = ind.size()-1
  gx.exec_menu("Shape Plot")
  scr[0].copy(tvec)
  scr[0].add(500*tstep)  # how many steps to keep it illuminated
  drv.fill(0)
  ii=jj=0 
  for tt in np.linspace(0,h.tstop,h.tstop/tstep+1):
    while ii < sz and tt > tvec.x[ii]:
      drv.x[int(animv[0].indwhere("==",ind.x[ii]))]=2
      ii = ii + 1
    while jj < sz and tt > scr[0].x[jj]:
      drv.x[int(animv[0].indwhere("==",ind.x[jj]))]=0
      jj = jj + 1
    gx.flush()
    h.doEvents()

#  run() and run sequences
h.cvode_active(1)
def run (): h.run()

def runseq ():
  createnet()
  setparams()
  savspks()
  run()

## typical use runs -- commented
'''
ncell=100
ncell=10
w=-1e-6 # minimal
runseq()
g = h.Graph()
g.erase_all
markspks()
showspks()
w=-0.3 # and repeat
setparams()
run() #   # don't repeat runseq() unless want to change # of cells -- just use run()
# animplot()
# anim()
'''
